@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +20,7 @@
 
 #include <memory>
 #include "associatedialog.h"
+#include "wraplibxml.h"
 
 BEGIN_EVENT_TABLE ( AssociateDialog, wxDialog )
 	EVT_BUTTON ( wxID_OK, AssociateDialog::OnOk )
@@ -36,6 +38,7 @@ AssociateDialog::AssociateDialog (
     const wxString& typeParameter,
     const wxString& extensionParameter,
     const wxString& urlParameter,
+    wxString *lastDir,
     bool auxNeededParameter,
     const wxString& auxLabelTextParameter,
     const wxString& auxParameter ) :
@@ -45,6 +48,7 @@ AssociateDialog::AssociateDialog (
 		type ( typeParameter ),
 		extension ( extensionParameter ),
 		url ( urlParameter ),
+		mLastDir ( lastDir ),
 		auxNeeded ( auxNeededParameter ),
 		auxLabelText ( auxLabelTextParameter ),
 		aux ( auxParameter )
@@ -158,30 +162,31 @@ void AssociateDialog::OnBrowse ( wxCommandEvent& e )
 {
 	wxString extensionArgument;
 	extensionArgument =
-	    type +
-	    _T ( " (" ) +
-	    extension +
-	    _T ( ")|" ) +
-	    extension +
-	    _ ( "|All files (*.*)|*.*" );
+	    type + _T ( " (" ) + extension + _T ( ")|" ) + extension
+	    + _ ( "|All files (*.*)|*.*" );
+	wxFileName fileName = WrapLibxml::URLToFileName ( urlCtrl->GetValue() );
+	wxString dir = fileName.GetPath();
+	if ( dir.empty() && mLastDir )
+		dir = *mLastDir;
 	std::auto_ptr<wxFileDialog> fd ( new wxFileDialog (
 	                                     this,
 	                                     _ ( "Select " ) + type,
-	                                     _T ( "" ),
+	                                     dir,
 	                                     _T ( "" ),
 	                                     extensionArgument,
 #if wxCHECK_VERSION(2,9,0)
-	                                     wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR
+	                                     wxFD_OPEN | wxFD_FILE_MUST_EXIST
 #else
-	                                     wxOPEN | wxFILE_MUST_EXIST | wxCHANGE_DIR
+	                                     wxOPEN | wxFILE_MUST_EXIST
 #endif
 	                                     ) );
 
 	if ( fd->ShowModal() == wxID_OK )
 	{
-		wxString newValue = fd->GetPath();
-		newValue.Replace ( _T ( "\\" ), _T ( "/" ), true );
-		newValue.Replace ( _T ( " " ), _T ( "%20" ), true );
+		wxString newValue = WrapLibxml::FileNameToURL ( fd->GetPath() );
 		urlCtrl->SetValue ( newValue );
+
+		if ( mLastDir )
+			*mLastDir = fd->GetDirectory();
 	}
 }
