@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +31,7 @@ BEGIN_EVENT_TABLE ( CommandPanel, wxPanel )
 	EVT_BUTTON ( ID_BUTTON_PATH, CommandPanel::OnVariableButton )
 	EVT_BUTTON ( ID_BUTTON_EXTENSION, CommandPanel::OnVariableButton )
 	EVT_BUTTON ( ID_BUTTON_FULLPATH, CommandPanel::OnVariableButton )
+	EVT_CHAR_HOOK ( CommandPanel::OnCharHook )
 END_EVENT_TABLE()
 
 CommandPanel::CommandPanel (
@@ -265,3 +267,11 @@ void CommandPanel::OnIdle ( wxIdleEvent& event )
 	outputInsert->Enable ( b );
 	outputNewDocument->Enable ( b );
 }
+
+void CommandPanel::OnCharHook ( wxKeyEvent& event )
+{
+	if ( event.GetKeyCode() == WXK_ESCAPE && event.GetModifiers() == 0 )
+		( ( MyFrame* ) GetParent() )->closeCommandPane();
+	else
+		event.Skip();
+}
