@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -158,7 +159,7 @@ void HouseStyle::collectRules ( const std::string& fileName,
 	xrr->getExcludeVector ( localExcludeVector );
 	for ( excludeIterator = localExcludeVector.begin();
 	        excludeIterator != localExcludeVector.end();
-	        excludeIterator++ )
+	        ++excludeIterator )
 		excludeSet.insert ( *excludeIterator );
 
 	* ( ruleCount ) += xrr->getRuleCount();
@@ -169,7 +170,7 @@ void HouseStyle::collectRules ( const std::string& fileName,
 	std::vector<std::string>::iterator includeIterator;
 	for ( includeIterator = includeVector.begin();
 	        includeIterator != includeVector.end();
-	        includeIterator++ )
+	        ++includeIterator )
 	{
 		if ( !excludeSet.count ( *includeIterator ) )
 			collectRules ( *includeIterator, ruleVector, excludeSet, ruleCount );
@@ -243,7 +244,7 @@ bool HouseStyle::createReport()
 
 			for ( matchIterator = contextVector.begin();
 				matchIterator != contextVector.end();
-				matchIterator++ )
+				++matchIterator )
 			{
 				matchIterator->report = "Not in dictionary";
 				matchIterator->elementCount = elementCount;	
@@ -269,7 +270,7 @@ bool HouseStyle::createReport()
 	
 					for ( matchIterator = contextVector.begin();
 						matchIterator != contextVector.end();
-						matchIterator++ )
+						++matchIterator )
 					{
 						if ( rule->getAdjustCaseAttribute() )
 							CaseHandler::adjustCase (
