@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,6 +34,7 @@ class LocationPanel : public wxPanel
 		void update (
 		    XmlDoc *docParameter = NULL,
 		    const wxString& parent = wxEmptyString );
+		void OnSysColourChanged ( wxSysColourChangedEvent &event );
 	private:
 		void indentStructure ( wxString& structure );
 		MyFrame *parentWindow;
