@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +26,6 @@
 #endif
 #include "styledialog.h"
 #include "nocasecompare.h"
-#define ngettext wxGetTranslation
 
 BEGIN_EVENT_TABLE ( StyleDialog, wxDialog )
 	EVT_BUTTON ( ID_STYLE_REPORT, StyleDialog::OnReport )
@@ -379,7 +379,7 @@ StyleDialog::StyleDialog (
 StyleDialog::~StyleDialog()
 {
 	std::set<wxString>::iterator it;
-	for ( it = tempFiles.begin(); it != tempFiles.end(); it++ )
+	for ( it = tempFiles.begin(); it != tempFiles.end(); ++it )
 		wxRemoveFile ( *it );
 }
 
@@ -484,7 +484,7 @@ void StyleDialog::OnReport ( wxCommandEvent& event )
 	std::string prelogUtf8, matchUtf8, postlogUtf8, replaceUtf8, reportUtf8;
 	wxString matchNo, prelog, match, postlog, replace, report;
 	int i = 0;
-	for ( it = matchVector.begin(); it != matchVector.end(); it++ )
+	for ( it = matchVector.begin(); it != matchVector.end(); ++it )
 	{
 		matchNo.Printf ( _T ( "%i" ), i + 1 ); // display numbers from 1
 		prelogUtf8 = flatWhiteSpace ( ( *it ).prelog );
@@ -508,7 +508,7 @@ void StyleDialog::OnReport ( wxCommandEvent& event )
 		++i;
 	}
 	wxString message;
-	message.Printf ( ngettext ( L"%i error", L"%i errors", i ), i );
+	message.Printf ( wxPLURAL ( "%i error", "%i errors", i ), i );
 	status->SetStatusText ( message );
 	if ( i )
 		table->SetFocus();
@@ -643,7 +643,7 @@ void StyleDialog::OnStyleWebReport ( wxCommandEvent& event )
 	ofs << "<td><b>Suggestion</b></td><td><b>Report</b></td></tr>";
 	std::vector<ContextMatch>::iterator it;
 	int matchCount = 0;
-	for ( it = v.begin(); it != v.end(); it++ )
+	for ( it = v.begin(); it != v.end(); ++it )
 	{
 		ofs << "<tr><td align=\"right\">";
 		ofs << ++matchCount;
@@ -681,7 +681,7 @@ void StyleDialog::OnStyleWebSummary ( wxCommandEvent& event )
 	for (
 	    vectorIterator = v.begin();
 	    vectorIterator != v.end();
-	    vectorIterator++ )
+	    ++vectorIterator )
 	{
 		if ( ( matchMap.find ( vectorIterator->match ) ) != matchMap.end() )
 			++ ( matchMap[vectorIterator->match] );
@@ -712,7 +712,7 @@ void StyleDialog::OnStyleWebSummary ( wxCommandEvent& event )
 	for (
 	    mapIterator = matchMap.begin();
 	    mapIterator != matchMap.end();
-	    mapIterator++ )
+	    ++mapIterator )
 	{
 		ofs << "<tr><td>";
 		ofs << we.xmliseTextNode ( mapIterator->first );
@@ -966,7 +966,7 @@ std::string StyleDialog::flatWhiteSpace ( std::string& s )
 	std::string::iterator it;
 	std::string output;
 	output.reserve ( s.size() );
-	for ( it = s.begin(); it != s.end(); it++ )
+	for ( it = s.begin(); it != s.end(); ++it )
 	{
 		if ( *it == '\t' || *it == '\r' || *it == '\n' )
 			output += ' ';
