@@ -1,3 +1,23 @@
+/*
+ * Copyright 2005-2009 Gerald Schmidt.
+ *
+ * This file is part of Xml Copy Editor.
+ *
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * Xml Copy Editor is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Xml Copy Editor; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #include <wx/wx.h>
 #include <wx/textbuf.h>
 #include "xmlctrl.h"
@@ -13,18 +33,18 @@ DEFINE_EVENT_TYPE(wxEVT_COMMAND_VALIDATION_COMPLETED);
 
 ValidationThread::ValidationThread (
 	wxEvtHandler *handler,
-	const char *buffer,
+	const char *utf8Buffer,
 	const wxString &system )
 	: wxThread ( wxTHREAD_JOINABLE )
 	, mStopping ( false )
 {
-	if ( buffer == NULL )
+	if ( utf8Buffer == NULL )
 	{
 		throw;
 	}
 
 	myEventHandler = handler;
-	myBuffer = buffer;
+	myBuffer = utf8Buffer;
 	mySystem = system;
 	myIsSucceeded = false;
 }
@@ -58,16 +78,8 @@ void *ValidationThread::Entry()
 
 	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
 
-	if ( myIsSucceeded )
-	{
-		myPosition = std::make_pair ( 0, 0 );
-		myMessage = wxEmptyString;
-	}
-	else
-	{
-		myPosition = validator->getErrorPosition();
-		myMessage = validator->getLastError();
-	}
+	myPosition = validator->getErrorPosition();
+	myMessage = validator->getLastError();
 
 	if ( !TestDestroy() )
 	{
