@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,10 +30,11 @@
 #include <wx/filesys.h>
 #include <wx/uri.h>
 
+// Convert wxString to const char *
 #ifdef __WXMSW__ // Libxml supports utf8 file name on windows
-#define CONV(url) ( ( const char * ) ( url ).utf8_str() )
+#define CONV(s) ( ( const char * ) ( s ).utf8_str() )
 #else
-#define CONV(url) ( ( const char * ) ( url ).mb_str ( wxConvLocal ) )
+#define CONV(s) ( ( const char * ) ( s ).mb_str ( wxConvLocal ) )
 #endif
 
 
@@ -49,8 +51,8 @@ public:
 		LIBXML_TEST_VERSION
 
 		xmlInitializeCatalog();
-		xmlLoadCatalog ( catalogPath.mb_str() );
-		::catalog = xmlLoadACatalog ( catalogPath.mb_str() );
+		xmlLoadCatalog ( CONV ( catalogPath ) );
+		::catalog = xmlLoadACatalog ( CONV ( catalogPath ) );
 
 		initGenericErrorDefaultFunc ( NULL );
 	}
@@ -112,7 +114,7 @@ WrapLibxml::~WrapLibxml()
 }
 
 bool WrapLibxml::validate ( const std::string& utf8DocBuf,
-		const wxString &docUrl )
+		const wxString &docFileName )
 {
 	output = "";
 
@@ -129,8 +131,10 @@ bool WrapLibxml::validate ( const std::string& utf8DocBuf,
 	int flags = XML_PARSE_DTDVALID;
 	if ( !netAccess )
 		flags |= XML_PARSE_NONET;
+	xmlChar *url = xmlFileNameToURL ( docFileName );
 	docPtr = xmlCtxtReadMemory ( ctxt, utf8DocBuf.c_str(), utf8DocBuf.length(),
-			CONV ( docUrl ), "UTF-8", flags);
+			( const char * ) url, "UTF-8", flags);
+	xmlFree ( url );
 
 	bool returnValue = docPtr != NULL && ctxt->valid != 0;
 
@@ -143,7 +147,7 @@ bool WrapLibxml::validate ( const std::string& utf8DocBuf,
 bool WrapLibxml::validateRelaxNG (
     const wxString &schemaFileName,
     const std::string &utf8DocBuf,
-    const wxString &docUrl )
+    const wxString &docFileName )
 {
 	output = "";
 
@@ -155,7 +159,9 @@ bool WrapLibxml::validateRelaxNG (
 	xmlRelaxNGPtr schemaPtr = NULL;
 
 	do {
-		rngParserCtxt = xmlRelaxNGNewParserCtxt ( CONV ( schemaFileName ) );
+		xmlChar *url = xmlFileNameToURL ( schemaFileName );
+		rngParserCtxt = xmlRelaxNGNewParserCtxt ( ( const char * ) url );
+		xmlFree ( url );
 		if ( rngParserCtxt == NULL )
 		{
 			nonParserError = _("Cannot create an RNG parser context");
@@ -181,8 +187,10 @@ bool WrapLibxml::validateRelaxNG (
 		int flags = XML_PARSE_DTDVALID;
 		if ( !netAccess )
 			flags |= XML_PARSE_NONET;
+		url = xmlFileNameToURL ( docFileName );
 		docPtr = xmlCtxtReadMemory ( ctxt, utf8DocBuf.c_str(),
-					utf8DocBuf.length(), CONV ( docUrl ), "UTF-8", flags );
+					utf8DocBuf.length(), ( const char * ) url, "UTF-8", flags );
+		xmlFree ( url );
 		if ( docPtr == NULL )
 			break;
 
@@ -202,7 +210,7 @@ bool WrapLibxml::validateRelaxNG (
 bool WrapLibxml::validateW3CSchema (
     const wxString &schemaFileName,
     const std::string &utf8DocBuf,
-    const wxString &docUrl )
+    const wxString &docFileName )
 {
 	output = "";
 
@@ -215,7 +223,9 @@ bool WrapLibxml::validateW3CSchema (
 	xmlSchemaPtr schemaPtr = NULL;
 
 	do {
-		rngParserCtxt = xmlSchemaNewParserCtxt ( CONV ( schemaFileName ) );
+		xmlChar *url = xmlFileNameToURL ( schemaFileName );
+		rngParserCtxt = xmlSchemaNewParserCtxt ( ( const char * ) url );
+		xmlFree ( url );
 		if ( rngParserCtxt == NULL )
 			return false;
 
@@ -239,8 +249,10 @@ bool WrapLibxml::validateW3CSchema (
 		int flags = XML_PARSE_DTDLOAD;
 		if ( !netAccess )
 			flags |= XML_PARSE_NONET;
+		url = xmlFileNameToURL ( docFileName );
 		docPtr = xmlCtxtReadMemory ( ctxt, utf8DocBuf.c_str(),
-					utf8DocBuf.length(), CONV ( docUrl ), "UTF-8", flags );
+					utf8DocBuf.length(), ( const char * ) url, "UTF-8", flags );
+		xmlFree ( url );
 		if ( docPtr == NULL )
 			break;
 
@@ -260,26 +272,26 @@ bool WrapLibxml::validateW3CSchema (
 
 bool WrapLibxml::parse (
     const std::string& utf8DocBuf,
-    const wxString &docUrl,
+    const wxString &docFileName,
     bool indent,
     bool resolveEntities )
 {
-	return parse ( utf8DocBuf.c_str(), utf8DocBuf.length(), docUrl,
+	return parse ( utf8DocBuf.c_str(), utf8DocBuf.length(), docFileName,
 			indent, resolveEntities );
 }
 
 bool WrapLibxml::parse (
-    const wxString &docUrl,
+    const wxString &docFileName,
     bool indent,
     bool resolveEntities )
 {
-	return parse ( NULL, 0, docUrl, indent, resolveEntities );
+	return parse ( NULL, 0, docFileName, indent, resolveEntities );
 }
 
 bool WrapLibxml::parse (
     const char *utf8DocBuf,
     size_t utf8DocBufSize,
-    const wxString &docUrl,
+    const wxString &docFileName,
     bool indent,
     bool resolveEntities )
 {
@@ -302,10 +314,14 @@ bool WrapLibxml::parse (
 		flags |= XML_PARSE_NONET;
 
 	if ( utf8DocBuf != NULL)
+	{
+		xmlChar *url = xmlFileNameToURL ( docFileName );
 		docPtr = xmlCtxtReadMemory ( ctxt, utf8DocBuf, utf8DocBufSize,
-					CONV ( docUrl ), "UTF-8", flags );
+					( const char * ) url, "UTF-8", flags );
+		xmlFree ( url );
+	}
 	else
-		docPtr = xmlCtxtReadFile ( ctxt, CONV ( docUrl ), NULL, flags );
+		docPtr = xmlCtxtReadFile ( ctxt, CONV ( docFileName ), NULL, flags );
 	if ( docPtr == NULL )
 	{
 		xmlFreeParserCtxt ( ctxt );
@@ -340,7 +356,7 @@ bool WrapLibxml::parse (
 }
 
 bool WrapLibxml::xpath ( const wxString &xpath, const std::string &utf8DocBuf,
-	    const wxString &docUrl )
+	    const wxString &docFileName )
 {
 	output = "";
 
@@ -356,16 +372,12 @@ bool WrapLibxml::xpath ( const wxString &xpath, const std::string &utf8DocBuf,
 		return false;
 	}
 
-	docPtr = xmlCtxtReadMemory (
-	             ctxt,
-	             utf8DocBuf.c_str(),
-	             utf8DocBuf.length(),
-	             CONV ( docUrl ),
-	             "UTF-8",
-	             //(netAccess) ? XML_PARSE_DTDLOAD | XML_PARSE_NOENT : XML_PARSE_DTDLOAD | XML_PARSE_NONET | XML_PARSE_NOENT
-	             XML_PARSE_NOENT | XML_PARSE_NONET | XML_PARSE_NSCLEAN
-	         );
-
+	//(netAccess) ? XML_PARSE_DTDLOAD | XML_PARSE_NOENT : XML_PARSE_DTDLOAD | XML_PARSE_NONET | XML_PARSE_NOENT
+	const static int flags = XML_PARSE_NOENT | XML_PARSE_NONET | XML_PARSE_NSCLEAN;
+	xmlChar *url = xmlFileNameToURL ( docFileName );
+	docPtr = xmlCtxtReadMemory ( ctxt, utf8DocBuf.c_str(), utf8DocBuf.length(),
+				( const char * ) url, "UTF-8", flags );
+	xmlFree ( url );
 	if ( docPtr == NULL )
 	{
 		xmlFreeParserCtxt ( ctxt );
@@ -374,7 +386,6 @@ bool WrapLibxml::xpath ( const wxString &xpath, const std::string &utf8DocBuf,
 
 	xmlXPathContextPtr context = NULL;
 	xmlXPathObjectPtr result = NULL;
-	xmlNodeSetPtr nodeset = NULL;
 
 	context = xmlXPathNewContext ( docPtr );
 	if ( !context )
@@ -395,28 +406,7 @@ bool WrapLibxml::xpath ( const wxString &xpath, const std::string &utf8DocBuf,
 
 	bool xpathIsValid = ( result ) ? true : false;
 
-	while ( result != NULL )
-	{
-		if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
-			break;
-		xmlBufferPtr bufferPtr = xmlBufferCreate();
-		if ( bufferPtr == NULL )
-			break;
-		nodeset = result->nodesetval;
-		for ( int i = 0; i < nodeset->nodeNr; i++ )
-		{
-			xmlNodePtr node = nodeset->nodeTab[i];
-			if ( !node )
-				break;
-			xmlNodeDump ( bufferPtr, NULL, node, 0, 1 );
-
-			output += ( const char * ) xmlBufferContent ( bufferPtr );
-			output += '\n';
-			xmlBufferEmpty ( bufferPtr );
-		}
-		xmlBufferFree ( bufferPtr );
-		break;
-	}
+	output = dumpXPathObject ( result );
 
 	xmlXPathFreeObject ( result );
 	xmlXPathFreeContext ( context );
@@ -426,29 +416,125 @@ bool WrapLibxml::xpath ( const wxString &xpath, const std::string &utf8DocBuf,
 	return xpathIsValid;
 }
 
-bool WrapLibxml::xslt (
-    const wxString &styleFileName,
-    const std::string &utf8DocBuf,
-    const wxString &docUrl
-)
+std::string WrapLibxml::dumpXPathObject ( xmlXPathObjectPtr obj )
 {
-	return xslt ( styleFileName, utf8DocBuf.c_str(), utf8DocBuf.length(),
-			docUrl );
+	std::stringstream sstream;
+	if ( !obj )
+		return sstream.str();
+
+	switch ( obj->type )
+	{
+	case XPATH_NODESET:
+	{
+		if ( xmlXPathNodeSetIsEmpty ( obj->nodesetval ) )
+			break;
+
+		xmlBufferPtr bufferPtr = xmlBufferCreate();
+		if ( bufferPtr == NULL )
+			break;
+
+		xmlNodeSetPtr nodeset = obj->nodesetval;
+		for ( int i = 0; i < nodeset->nodeNr; i++ )
+		{
+			xmlNodePtr node = nodeset->nodeTab[i];
+			if ( !node )
+				break;
+			xmlNodeDump ( bufferPtr, NULL, node, 0/*level*/, TRUE/*format*/ );
+
+			sstream << ( const char * ) xmlBufferContent ( bufferPtr )
+					<< '\n';
+			xmlBufferEmpty ( bufferPtr );
+		}
+		xmlBufferFree ( bufferPtr );
+		break;
+	}
+
+	case XPATH_BOOLEAN:
+		sstream << !!obj->boolval;
+		break;
+
+	case XPATH_NUMBER:
+		switch ( xmlXPathIsInf ( obj->floatval ) )
+		{
+		case 1:
+			sstream << wxString ( _("Infinity") ).utf8_str();
+			break;
+		case -1:
+			sstream << wxString ( _("-Infinity") ).utf8_str();
+			break;
+		default:
+			if ( xmlXPathIsNaN ( obj->floatval ) )
+				sstream << wxString ( _("NaN") ).utf8_str();
+			else
+				sstream << obj->floatval;
+			break;
+		}
+		break;
+
+	case XPATH_STRING:
+		sstream << obj->stringval;
+		break;
+
+	case XPATH_POINT:
+		xmlBufferPtr bufferPtr;
+		bufferPtr = xmlBufferCreate();
+		if ( bufferPtr == NULL )
+			break;
+		xmlNodeDump ( bufferPtr, NULL, ( xmlNodePtr ) obj->user,
+				1/*level*/, TRUE/*format*/ );
+		sstream << xmlBufferContent ( bufferPtr );
+		xmlBufferFree ( bufferPtr );
+		break;
+
+	case XPATH_RANGE:
+	case XPATH_LOCATIONSET:
+	case XPATH_XSLT_TREE:
+	default:
+		FILE *fp = tmpfile();
+		xmlXPathDebugDumpObject ( fp, obj, 0/*depth*/ );
+
+		std::string str;
+		long size = ftell ( fp );
+		if ( size == -1 )
+			return str;
+
+		str.resize ( size + 1 );
+		if ( str.capacity() != ( size_t ) size + 1 )
+			return str;
+		rewind ( fp );
+		size_t read = fread ( ( char * ) str.c_str(), 1, str.capacity(), fp );
+		str[size] = '\0';
+		fclose ( fp );
+
+		return read == str.capacity() ? str : sstream.str();
+	}
+
+	return sstream.str();
 }
 
 bool WrapLibxml::xslt (
     const wxString &styleFileName,
-    const wxString &docUrl
+    const std::string &utf8DocBuf,
+    const wxString &docFileName
 )
 {
-	return xslt ( styleFileName, NULL, 0, docUrl );
+	return xslt ( styleFileName, utf8DocBuf.c_str(), utf8DocBuf.length(),
+			docFileName );
+}
+
+bool WrapLibxml::xslt (
+    const wxString &styleFileName,
+    const wxString &docFileName
+)
+{
+	return xslt ( styleFileName, NULL, 0, docFileName );
 }
 
 bool WrapLibxml::xslt (
     const wxString &styleFileName,
     const char *utf8DocBuf,
     size_t utf8DocBufSize,
-    const wxString &docUrl
+    const wxString &docFileName
 )
 {
 	output = "";
@@ -479,10 +565,14 @@ bool WrapLibxml::xslt (
 		if ( !netAccess )
 			flags |= XML_PARSE_NONET;
 		if ( utf8DocBuf != NULL )
+		{
+			xmlChar *url = xmlFileNameToURL ( docFileName );
 			doc = xmlCtxtReadMemory ( ctxt, utf8DocBuf, utf8DocBufSize,
-						CONV ( docUrl ), "UTF-8", flags );
+					( const char * ) url, "UTF-8", flags );
+			xmlFree ( url );
+		}
 		else
-			doc = xmlCtxtReadFile ( ctxt,  CONV ( docUrl ), NULL, flags );
+			doc = xmlCtxtReadFile ( ctxt, CONV ( docFileName ), NULL, flags );
 		if ( !doc )
 			break;
 
@@ -579,8 +669,12 @@ int WrapLibxml::saveEncoding (
 	if ( !netAccess )
 		flags |= XML_PARSE_NONET;
 	if ( utf8Buffer != NULL )
+	{
+		xmlChar *url = xmlFileNameToURL ( fileNameSource );
 		docPtr = xmlCtxtReadMemory ( ctxt, utf8Buffer, utf8BufferSize,
-				CONV ( fileNameSource ), "UTF-8", flags );
+				( const char * ) url, "UTF-8", flags );
+		xmlFree ( url );
+	}
 	else
 		docPtr = xmlCtxtReadFile ( ctxt, CONV ( fileNameSource ), NULL, flags );
 	if ( !docPtr )
@@ -671,9 +765,74 @@ wxString WrapLibxml::catalogResolve
 	wxString url ( s, wxConvUTF8 );
 	xmlFree ( s );
 
-	wxFileName file = wxFileSystem::URLToFileName ( url );
+	wxFileName file = URLToFileName ( url );
 	if ( file.IsFileReadable() )
 		return file.GetFullPath();
 
 	return url;
 }
+
+wxString WrapLibxml::FileNameToURL ( const wxString &fileName )
+{
+	xmlChar *s = xmlFileNameToURL ( fileName );
+	if ( !s )
+		return wxEmptyString;
+
+	wxString url = wxString::FromUTF8 ( ( char * ) s );
+	xmlFree ( s );
+
+	return url;
+}
+
+xmlChar *WrapLibxml::xmlFileNameToURL ( const wxString &fileName )
+{
+	if ( fileName.empty() )
+		return NULL;
+
+	wxFileName fn ( fileName );
+	fn.Normalize();
+	wxString url = fn.GetFullPath();
+
+	return xmlPathToURI ( ( xmlChar * ) ( const char * ) url.utf8_str() );
+}
+
+wxFileName WrapLibxml::URLToFileName ( const wxString &url )
+{
+#if wxCHECK_VERSION(2,9,0)
+	return wxFileSystem::URLToFileName ( url );
+#else
+	xmlURIPtr uri = xmlParseURI ( url.utf8_str() );
+	if ( !uri )
+		return wxFileName ( url );
+
+	do {
+		if ( uri->scheme != NULL && strcmp (uri->scheme, "file" ) )
+			break;
+#ifdef _MSC_VER
+		if ( uri->server && stricmp ( uri->server, "localhost") )
+			break;
+#else
+		if ( uri->server && strcasecmp ( uri->server, "localhost") )
+			break;
+#endif
+		if ( uri->path == NULL || !*uri->path )
+			break;
+
+		char *path = uri->path;
+		// Does it begin with "/C:" ?
+		if ( *path == '/' && wxIsalpha ( path[1] ) && path[2] == ':')
+			path++;
+
+		wxFileName file ( wxString ( path, wxConvUTF8 ) );
+
+		xmlFreeURI ( uri );
+
+		return file;
+
+	} while ( false );
+
+	xmlFreeURI ( uri );
+
+	return wxFileName(url);
+#endif // wxCHECK_VERSION(2,9,0)
+}
