@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,11 +31,13 @@
 #include <libxml/catalog.h>
 #include <libxml/xmlreader.h>
 #include <libxml/xmlmemory.h>
+#include <libxml/uri.h>
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include <wx/wx.h>
+#include <wx/filename.h>
 
 class WrapLibxml
 {
@@ -43,18 +46,20 @@ class WrapLibxml
 
 		WrapLibxml ( bool netAccessParameter = false );
 		virtual ~WrapLibxml();
-		bool validate ( const std::string &utf8DocBuf, const wxString &docUrl );
+		bool validate (
+		    const std::string &utf8DocBuf,
+		    const wxString &docFileName );
 		bool validateRelaxNG (
 		    const wxString &schemaFileName,
 		    const std::string &utf8DocBuf,
-		    const wxString &docUrl );
+		    const wxString &docFileName );
 		bool validateW3CSchema (
 		    const wxString &schemaFileName,
 		    const std::string &utf8DocBuf,
-		    const wxString &docUrl );
+		    const wxString &docFileName );
 		bool parse (
 		    const std::string &utf8DocBuf,
-		    const wxString &docUrl,
+		    const wxString &docFileName,
 		    bool indent = false,
 		    bool resolveEntities = false );
 		bool parse (
@@ -68,16 +73,23 @@ class WrapLibxml
 		    bool indent = false,
 		    bool resolveEntities = false );
 		bool bufferWellFormed ( const std::string& utf8Buffer );
-		bool xpath ( const wxString &xpath, const std::string &utf8DocBuf,
-		    const wxString &docUrl );
-		bool xslt ( const wxString &styleFileName,
+		bool xpath (
+		    const wxString &xpath,
 		    const std::string &utf8DocBuf,
-		    const wxString &docUrl );
-		bool xslt ( const wxString &styleFileName, const wxString &docUrl );
-		bool xslt ( const wxString &styleFileName,
+		    const wxString &docFileName );
+		std::string dumpXPathObject ( xmlXPathObjectPtr obj );
+		bool xslt (
+		    const wxString &styleFileName,
+		    const std::string &utf8DocBuf,
+		    const wxString &docFileName );
+		bool xslt (
+		    const wxString &styleFileName,
+		    const wxString &docFileName );
+		bool xslt (
+		    const wxString &styleFileName,
 		    const char *utf8DocBuf,
 		    size_t utf8DocBufSize,
-		    const wxString &docUrl );
+		    const wxString &docFileName );
 		wxString getLastError();
 		std::pair<int, int> getErrorPosition();
 		std::string getOutput();
@@ -101,6 +113,10 @@ class WrapLibxml
 		wxString catalogResolve (
 		    const wxString &publicId,
 		    const wxString &systemId );
+
+		static wxString FileNameToURL ( const wxString &fileName );
+		static xmlChar *xmlFileNameToURL ( const wxString &fileName );
+		static wxFileName URLToFileName ( const wxString &url );
 	private:
 		bool netAccess;
 		std::string output;
