@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,8 +31,9 @@
 
 struct AssociateXsdData : public ParserData
 {
-	std::string buffer, path, namespaceAttribute;
-	bool rootElementSeen;
+	std::string buffer;//, namespaceAttribute;
+	wxString path;
+	size_t depth;
 	std::map<std::string, std::string> namespaceMap;
 };
 
@@ -39,7 +41,8 @@ class XmlAssociateXsd : public WrapExpat
 {
 	public:
 		XmlAssociateXsd (
-		    const std::string& path = "",
+		    const wxString &path = wxEmptyString,
+		    const char *encoding = NULL,
 		    size_t size = BUFSIZ );
 		virtual ~XmlAssociateXsd();
 		std::string getBuffer()
