@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,19 +26,22 @@
 #include <expat.h>
 #include <string>
 #include <memory>
-#include "wrapexpat.h"
+#include "xmltextinfo.h"
 
 struct XslData : public ParserData
 {
-	std::string buffer, path;
-	bool rootElementSeen;
+	std::string buffer;
+	wxString path;
+	bool associated;
 };
 
 class XmlAssociateXsl : public WrapExpat
 {
 	public:
 		XmlAssociateXsl (
-		    const std::string& path = "",
+		    const XmlTextInfo &info,
+		    const wxString &path = wxEmptyString,
+		    const char *encoding = NULL,
 		    size_t size = BUFSIZ );
 		virtual ~XmlAssociateXsl();
 		std::string getBuffer()
@@ -46,7 +50,7 @@ class XmlAssociateXsl : public WrapExpat
 		}
 	private:
 		std::auto_ptr<XslData> d;
-		std::string encoding;
+		const XmlTextInfo &mInfo;
 		static void XMLCALL start (
 		    void *data,
 		    const XML_Char *el,
