@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -76,7 +77,9 @@ enum
 	ID_NEXT_DOCUMENT,
 	ID_OPTIONS,
 	ID_IMPORT_MSWORD,
-	ID_HIDE_PANE,
+	ID_CLOSE_MESSAGE_PANE,
+	ID_CLOSE_FIND_REPLACE_PANE,
+	ID_CLOSE_COMMAND_PANE,
 	ID_HOME,
 	ID_PASTE_NEW_DOCUMENT,
 	ID_PRINT_SETUP,
@@ -107,6 +110,7 @@ enum
 	ID_FIND,
 	ID_FIND_AGAIN,
 	ID_GOTO,
+	ID_TOGGLE_COMMENT,
 	ID_PRINT,
 	ID_WORD_COUNT,
 	ID_PRINT_PREVIEW,
@@ -122,6 +126,7 @@ enum
 	ID_CREATE_SCHEMA,
 	ID_DTD_TO_SCHEMA,
 	ID_XPATH,
+	ID_COPY_XPATH,
 	ID_XSLT,
 	ID_XSLT_TEI_FO,
 	ID_XSLT_TEI_HTML,
@@ -218,7 +223,9 @@ class MyFrame : public wxFrame
 		void OnCheckWellformedness ( wxCommandEvent& event );
 		void OnClose ( wxCommandEvent& event );
 		void OnCloseAll ( wxCommandEvent& event );
-		void OnClosePane ( wxCommandEvent& event );
+		void OnCloseMessagePane ( wxCommandEvent& event );
+		void OnCloseFindReplacePane ( wxCommandEvent& event );
+		void OnCloseCommandPane ( wxCommandEvent& event );
 		void OnCut ( wxCommandEvent& event );
 		void OnCopy ( wxCommandEvent& event );
 		void OnPaste ( wxCommandEvent& event );
@@ -231,6 +238,7 @@ class MyFrame : public wxFrame
 		void OnFindReplace ( wxCommandEvent& event );
 		void OnCommand ( wxCommandEvent& event );
 		void OnGlobalReplace ( wxCommandEvent& event );
+		void OnToggleComment ( wxCommandEvent& event );
 		void OnWordCount ( wxCommandEvent& event );
 		void OnFeedback ( wxCommandEvent& event );
 		void OnSplitTab ( wxCommandEvent& event );
@@ -270,7 +278,9 @@ class MyFrame : public wxFrame
 		void OnDialogReplaceAll ( wxFindDialogEvent& event );
 		void OnFrameClose ( wxCloseEvent& event );
 		void OnIdle ( wxIdleEvent& event );
-		void OnUpdateClosePane ( wxUpdateUIEvent& event );
+		void OnUpdateCloseMessagePane ( wxUpdateUIEvent& event );
+		void OnUpdateCloseFindReplacePane ( wxUpdateUIEvent& event );
+		void OnUpdateCloseCommandPane ( wxUpdateUIEvent& event );
 		void OnUpdateCloseAll ( wxUpdateUIEvent& event );
 		void OnUpdateUndo ( wxUpdateUIEvent& event );
 		void OnUpdatePreviousDocument ( wxUpdateUIEvent& event );
@@ -280,6 +290,7 @@ class MyFrame : public wxFrame
 		void OnUpdateCutCopy ( wxUpdateUIEvent& event );
 		void OnUpdateFindAgain ( wxUpdateUIEvent& event );
 		void OnUpdatePaste ( wxUpdateUIEvent& event );
+		void OnUpdateToggleComment ( wxUpdateUIEvent& event );
 		void OnUpdateDocRange ( wxUpdateUIEvent& event );
 		void OnUpdateReplaceRange ( wxUpdateUIEvent& event );
 		void OnUpdateReload ( wxUpdateUIEvent& event );
@@ -310,12 +321,15 @@ class MyFrame : public wxFrame
 #ifdef __WXMSW__
 		void OnDropFiles ( wxDropFilesEvent& event );
 #endif
+		void OnPromptGenerated ( wxNotifyEvent &event );
+		void OnCopyXPath ( wxCommandEvent &event );
+		void OnUpdateCopyXPath ( wxUpdateUIEvent& event );
 		void setStrictScrolling ( bool b );
 		void validateRelaxNG (
 		    XmlDoc *doc,
 		    const wxString& schemaName,
 		    wxString& fileName );
-		void closePane();
+		void closeMessagePane();
 		void closeFindReplacePane();
 		void closeCommandPane();
 		bool closeActiveDocument();
@@ -340,11 +354,12 @@ class MyFrame : public wxFrame
 		                   bool forcePane = false );
 
 		// public to allow IPC access
-		bool openFile ( wxString& fileName, bool largeFile = false );
+		bool openFile ( const wxString &fileName, bool largeFile = false );
 		bool isOpen ( const wxString& fileName );
 		bool activateTab ( const wxString& fileName );
 		void reloadTab();
 		void addToFileQueue ( wxString& fileName );
+		void openRememberedTabs();
 	private:
 		wxAuiManager manager;
 		wxFileConfig *config; // owned by MyApp
@@ -436,6 +451,7 @@ class MyFrame : public wxFrame
 		  exportStylesheet,
 		  exportFolder,
 		  lastSymbol;
+		wxString mLastDir;
 		bool globalReplaceAllDocuments,
 		toolbarVisible,
 		protectTags,
@@ -527,7 +543,6 @@ class MyFrame : public wxFrame
 		void showTopBars ( bool b );
 		void modifiedMessage();
 		void loadBitmaps();
-		void openRememberedTabs();
 		void getRawText ( XmlDoc *doc, std::string& buffer );
 		void updateToolbar();
 		std::string getApproximateEncoding ( char *docBuffer, size_t docBufferLen );
