@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +42,8 @@ BEGIN_EVENT_TABLE ( XmlCtrl, wxStyledTextCtrl )
 	EVT_RIGHT_UP ( XmlCtrl::OnMouseRightUp )
 	EVT_MIDDLE_DOWN ( XmlCtrl::OnMiddleDown )
 	EVT_COMMAND(wxID_ANY, wxEVT_COMMAND_VALIDATION_COMPLETED, XmlCtrl::OnValidationCompleted)
+	EVT_NOTIFY ( myEVT_NOTIFY_PROMPT_GENERATED, wxID_ANY, XmlCtrl::OnPromptGenerated )
+	EVT_KILL_FOCUS ( XmlCtrl::OnKillFocus )
 END_EVENT_TABLE()
 
 // global protection for validation threads
@@ -69,6 +72,7 @@ XmlCtrl::XmlCtrl (
 	, auxPath ( auxPathParameter )
 {
 	validationThread = NULL;
+	mPromptGeneratorThread = NULL;
 
 	grammarFound = false;
 	validationRequired = (buffer) ? true : false; // NULL for plain XML template
@@ -148,6 +152,8 @@ XmlCtrl::~XmlCtrl()
 		//validationThread->Delete();
 		//delete validationThread;
 	}
+	if ( mPromptGeneratorThread != NULL )
+		mPromptGeneratorThread->PendingDelete();
 }
 
 
@@ -490,7 +496,7 @@ void XmlCtrl::handleOpenAngleBracket ( wxKeyEvent& event )
 	wxString choice;
 	std::set<wxString> &childSet = elementMap[parent];
 	std::set<wxString>::iterator it;
-	for ( it = childSet.begin(); it != childSet.end(); it++ )
+	for ( it = childSet.begin(); it != childSet.end(); ++it )
 	{
 		if ( !choice.empty() )
 			choice.append ( _T ( "<" ) );
@@ -591,7 +597,7 @@ void XmlCtrl::handleEquals ( wxKeyEvent& event )
 	int cutoff = BUFSIZ;
 	for ( valueSetIterator = valueSet.begin();
 	        valueSetIterator != valueSet.end();
-	        valueSetIterator++ )
+	        ++valueSetIterator )
 	{
 		if ( ! ( cutoff-- ) )
 			break;
@@ -657,7 +663,7 @@ void XmlCtrl::handleSpace ( wxKeyEvent& event )
 	wxString tag = GetTextRange ( tagStartPos, pos );
 	std::map<wxString, std::set<wxString> > &curAttMap = attributeMap[elementName];
 	std::map<wxString, std::set<wxString> >::iterator it;
-	for ( it = curAttMap.begin(); it != curAttMap.end(); it++ )
+	for ( it = curAttMap.begin(); it != curAttMap.end(); ++it )
 	{
 		// avoid duplicate attributes
 		if ( tag.Contains ( it->first + _T ( "=" ) ) )
@@ -700,7 +706,7 @@ void XmlCtrl::handleAmpersand ( wxKeyEvent& event )
 		std::set<wxString>::iterator it = entitySet.begin();
 		choice += *it;
 		choice += _T ( ";" );
-		for ( it++; it != entitySet.end(); it++ )
+		for ( it++; it != entitySet.end(); ++it )
 		{
 			choice += _T ( "<" );
 			choice += *it;
@@ -973,40 +979,102 @@ wxString XmlCtrl::getLastAttributeName ( int pos )
 	return GetTextRange ( startPos + 1, pos );
 }
 
-int XmlCtrl::getParentCloseAngleBracket ( int pos, int range )
+int XmlCtrl::findNextEndTag (
+		int pos,
+		unsigned depth /*= 1*/,
+		int character /*= '>'*/,
+		int range /*= USHRT_MAX * 4*/ )
 {
-	int cutoff, iteratorPos, depth;
-	cutoff = ( ( pos - range ) > 2 ) ? pos - range : 2;
-	depth = 1;
-	for (
-	    iteratorPos = pos;
-	    iteratorPos > cutoff;
-	    --iteratorPos )
-	{
-		int type, style;
-		style = getLexerStyleAt ( iteratorPos );
+	wxASSERT ( character == '<' || character == '>' );
 
-		if ( GetCharAt ( iteratorPos ) == '>' &&
-		        ( style == wxSTC_H_TAG ||
-		          style == wxSTC_H_TAGUNKNOWN ) )
+	unsigned openAngleBrackets = 0;
+	int cutoff = pos + range;
+	if ( cutoff > GetEndStyled() )
+		cutoff = GetEndStyled();
+	for ( ; pos < cutoff; ++pos )
+	{
+		int ch = GetCharAt ( pos );
+		if ( ch == '<' )
+			openAngleBrackets = ( openAngleBrackets << 1 ) | 1;// Just a flag
+		// Check for empty tags, which have start tags but no end tags
+		if ( character != '>' || !openAngleBrackets )
+			if ( ch == '>' && getLexerStyleAt ( pos ) == wxSTC_H_TAGEND )
+				--depth;
+		if ( ch != character )
+			continue;
+
+		int style = getLexerStyleAt ( pos );
+		if ( style == wxSTC_H_TAG || style == wxSTC_H_TAGUNKNOWN )
 		{
-			type = getTagType ( iteratorPos );
+			int type = getTagType ( pos );
 			switch ( type )
 			{
-				case ( TAG_TYPE_CLOSE ) :
-								++depth;
-					break;
-				case ( TAG_TYPE_OPEN ) :
-								--depth;
-					break;
-				case ( TAG_TYPE_EMPTY ) :
-							case ( TAG_TYPE_OTHER ) :
-								case ( TAG_TYPE_ERROR ) :
-										break;
+			case TAG_TYPE_CLOSE:
+				--depth;
+				break;
+			case TAG_TYPE_OPEN:
+				// In case that the cursor is inside a start tag
+				if ( character != '>' || openAngleBrackets > 0 )
+					++depth;
+				break;
+			case TAG_TYPE_EMPTY:
+			case TAG_TYPE_OTHER:
+			case TAG_TYPE_ERROR:
+				break;
 			}
-			if ( !depth )
-				return iteratorPos;
 		}
+
+		if ( !depth )
+			return pos + 1;
+	}
+	return -1;
+}
+
+int XmlCtrl::findPreviousStartTag (
+		int pos,
+		unsigned depth /*= 1*/,
+		int character /*= '<'*/,
+		int range /*= USHRT_MAX * 4*/ )
+{
+	wxASSERT ( character == '<' || character == '>' );
+
+	int cutoff = ( ( pos - range ) > 2 ) ? pos - range : 2;
+	unsigned closeAngleBrackets = 0;
+	while ( pos-- > cutoff )
+	{
+		int ch = GetCharAt ( pos );
+		if ( ch == '>' )
+		{
+			closeAngleBrackets = ( closeAngleBrackets << 1 ) | 1;// Just a flag
+			// Check for empty tags, which have start tags but no end tags
+			if ( character != '>' && getLexerStyleAt ( pos ) == wxSTC_H_TAGEND )
+				++depth;
+		}
+		if ( ch != character )
+			continue;
+
+		int style = getLexerStyleAt ( pos );
+		if ( style == wxSTC_H_TAG || style == wxSTC_H_TAGUNKNOWN )
+		{
+			int type = getTagType ( pos );
+			switch ( type )
+			{
+			case TAG_TYPE_CLOSE:
+				// If the cursor is already in an end tag
+				if ( character != '<' || closeAngleBrackets > 0 )
+					++depth;
+				break;
+			case TAG_TYPE_OPEN:
+				--depth;
+				break;
+			case TAG_TYPE_EMPTY:
+			case TAG_TYPE_OTHER:
+			case TAG_TYPE_ERROR:
+				break;
+			}
+		}
+		if ( !depth )
+			return pos;
 	}
 	return -1;
 }
@@ -1034,34 +1102,58 @@ void XmlCtrl::adjustNoColumnWidth()
 
 void XmlCtrl::updatePromptMaps()
 {
-	wxString buffer = GetText();
-	std::string bufferUtf8;
-	bufferUtf8 = ( const char * ) buffer.mb_str ( wxConvUTF8 );
-	XmlEncodingHandler::setUtf8 ( bufferUtf8, true );
+	std::string bufferUtf8 = myGetTextRaw();
 
 	updatePromptMaps ( bufferUtf8.c_str(), bufferUtf8.size() );
 }
 
-void XmlCtrl::updatePromptMaps ( const char *buffer, size_t bufferLen )
+void XmlCtrl::updatePromptMaps ( const char *utf8Buffer, size_t bufferLen )
 {
 	attributeMap.clear();
 	elementMap.clear();
 	elementStructureMap.clear();
-	std::auto_ptr<XmlPromptGenerator> xpg ( new XmlPromptGenerator (
-	                                            basePath,
-	                                            auxPath ) );
-	xpg->parse ( buffer, bufferLen );
-	xpg->getAttributeMap ( attributeMap );
-	xpg->getRequiredAttributeMap ( requiredAttributeMap );
-	xpg->getElementMap ( elementMap );
-	xpg->getElementStructureMap ( elementStructureMap );
-	xpg->getEntitySet ( entitySet );
-	grammarFound = xpg->getGrammarFound();
+
+	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
+
+	if ( mPromptGeneratorThread )
+		return;
+
+	mPromptGeneratorThread = new XmlPromptGenerator (
+		GetEventHandler(), utf8Buffer, bufferLen, basePath, auxPath, "UTF-8"
+	);
+
+	if ( mPromptGeneratorThread->Create() != wxTHREAD_NO_ERROR
+	    || mPromptGeneratorThread->Run() != wxTHREAD_NO_ERROR )
+	{
+		delete mPromptGeneratorThread;
+		mPromptGeneratorThread = NULL;
+	}
+}
+
+void XmlCtrl::OnPromptGenerated ( wxNotifyEvent &event )
+{
+	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
+
+	if ( mPromptGeneratorThread == NULL )
+		return;
+
+	attributeMap = mPromptGeneratorThread->getAttributeMap();
+	requiredAttributeMap = mPromptGeneratorThread->getRequiredAttributeMap();
+	elementMap = mPromptGeneratorThread->getElementMap();
+	elementStructureMap = mPromptGeneratorThread->getElementStructureMap();
+	entitySet = mPromptGeneratorThread->getEntitySet();
+	grammarFound = mPromptGeneratorThread->getGrammarFound();
 	entitySet.insert ( _T ( "amp" ) );
 	entitySet.insert ( _T ( "apos" ) );
 	entitySet.insert ( _T ( "quot" ) );
 	entitySet.insert ( _T ( "lt" ) );
 	entitySet.insert ( _T ( "gt" ) );
+
+	mPromptGeneratorThread->Wait();
+	delete mPromptGeneratorThread;
+	mPromptGeneratorThread = NULL;
+
+	wxPostEvent ( GetParent()->GetEventHandler(), event );
 }
 
 void XmlCtrl::applyProperties (
@@ -1119,13 +1211,15 @@ void XmlCtrl::applyProperties (
 		SetMarginWidth ( 2, 16 );
 
 		// define folding markers
-		MarkerDefine ( wxSTC_MARKNUM_FOLDEREND, wxSTC_MARK_BOXPLUSCONNECTED,  *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDEROPENMID, wxSTC_MARK_BOXMINUSCONNECTED, *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDERMIDTAIL, wxSTC_MARK_TCORNER, *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDERTAIL, wxSTC_MARK_LCORNER, *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDERSUB, wxSTC_MARK_VLINE, *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDER, wxSTC_MARK_BOXPLUS, *wxWHITE, *wxBLACK );
-		MarkerDefine ( wxSTC_MARKNUM_FOLDEROPEN, wxSTC_MARK_BOXMINUS, *wxWHITE, *wxBLACK );
+		wxColor clrWnd = wxSystemSettings::GetColour ( wxSYS_COLOUR_WINDOW );
+		wxColor clrText = wxSystemSettings::GetColour ( wxSYS_COLOUR_WINDOWTEXT );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDEREND, wxSTC_MARK_BOXPLUSCONNECTED, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDEROPENMID, wxSTC_MARK_BOXMINUSCONNECTED, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDERMIDTAIL, wxSTC_MARK_TCORNER, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDERTAIL, wxSTC_MARK_LCORNER, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDERSUB, wxSTC_MARK_VLINE, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDER, wxSTC_MARK_BOXPLUS, clrWnd, clrText );
+		MarkerDefine ( wxSTC_MARKNUM_FOLDEROPEN, wxSTC_MARK_BOXMINUS, clrWnd, clrText );
 	}
 	else
 	{
@@ -1651,6 +1745,9 @@ void XmlCtrl::setColorScheme ( int scheme )
 		StyleSetBold ( wxSTC_H_SGML_DEFAULT, true );
 	}
 
+	wxColour color = wxSystemSettings::GetColour ( wxSYS_COLOUR_WINDOWTEXT );
+	StyleSetForeground ( wxSTC_STYLE_LINENUMBER, color );
+
 	applyVisibilityState ( visibilityState );
 }
 
@@ -1786,7 +1883,7 @@ wxString XmlCtrl::getOpenTag ( const wxString& element )
 	requiredAttributeSet = requiredAttributeMap[element];
 	if ( !requiredAttributeSet.empty() )
 	{
-		for ( it = requiredAttributeSet.begin(); it != requiredAttributeSet.end(); it++ )
+		for ( it = requiredAttributeSet.begin(); it != requiredAttributeSet.end(); ++it )
 		{
 			openTag += _T ( " " );
 			openTag += *it;
@@ -1954,8 +2051,6 @@ bool XmlCtrl::backgroundValidate()
 
 	std::string bufferUtf8 = myGetTextRaw();
 
-	XmlEncodingHandler::setUtf8( bufferUtf8, true );
-	
 	return backgroundValidate (
 		bufferUtf8.c_str(),
 		basePath,
@@ -2126,3 +2221,185 @@ void XmlCtrl::OnMiddleDown ( wxMouseEvent& event )
 	SetSelection ( pastePosition, pastePosition );
 	Paste();
 }
+
+bool XmlCtrl::selectCurrentElement()
+{
+	if ( type != FILE_TYPE_XML )
+		return false;
+
+	Colourise ( 0, -1 );
+
+	int pos = GetCurrentPos();
+	int style = getLexerStyleAt ( pos ) ;
+	if ( style == wxSTC_H_COMMENT )
+	{
+		int i = pos;
+		while ( --i >= 0 && getLexerStyleAt ( i ) == wxSTC_H_COMMENT )
+			continue;
+		SetSelectionStart ( i + 1 );
+
+		int styled = GetEndStyled();
+		i = pos;
+		while ( i < styled && getLexerStyleAt ( i ) == wxSTC_H_COMMENT )
+			i++;
+		SetSelectionEnd ( i );
+	}
+	else
+	{
+		// Select current tag
+		int start = findPreviousStartTag ( pos, 1, '<', pos );
+		if ( start < 0 )
+		{
+			MyFrame *frame = ( MyFrame * ) wxTheApp->GetTopWindow();
+			frame->statusProgress ( _("Cannot find the start tag") );
+			return false;
+		}
+		int range = GetTextLength() - pos;
+		int end = findNextEndTag ( pos, 1, '>', range );
+		if ( end < 0 )
+		{
+			MyFrame *frame = ( MyFrame * ) wxTheApp->GetTopWindow();
+			frame->statusProgress ( _("Cannot find the end tag") );
+			return false;
+		}
+		SetSelection ( start, end );
+	}
+
+	return true;
+}
+
+void XmlCtrl::toggleComment()
+{
+	MyFrame *frame = ( MyFrame * ) wxTheApp->GetTopWindow();
+	frame->statusProgress ( wxEmptyString );
+
+	int pos = -1;
+	wxString commentStart = _T ( "<!--" );
+	wxString commentEnd = _T ( "-->" );
+
+	// Is there a selection?
+	int from = GetSelectionStart();
+	int to = GetSelectionEnd();
+	switch ( type )
+	{
+	case FILE_TYPE_BINARY:
+		return;
+
+	case FILE_TYPE_CSS:
+		if ( from == to )
+			return;
+
+		commentStart = _T ( "/*" );
+		commentEnd = _T ( "*/" );
+		break;
+
+	case FILE_TYPE_XML:
+		if ( from != to )
+			break;
+
+		// Select current element
+		pos = GetCurrentPos();
+		if ( !selectCurrentElement() )
+			return;
+		break;
+
+	default:
+		if ( from == to )
+			return;
+		break;
+	}
+
+	wxString text = GetSelectedText();
+	wxASSERT ( !text.IsEmpty() );
+
+	// Skip leading spaces
+	wxString::iterator itr, start, end;
+	itr = start = text.begin();
+	end = text.end();
+	while ( itr != end && wxIsspace ( *itr ) )
+		++itr;
+
+	size_t startPos = itr - start;
+	int ret = text.compare ( startPos, commentStart.length(), commentStart );
+	if ( ret == 0 )
+	{
+		start = itr;
+		itr = end;
+		do {
+			--itr;
+		} while ( itr != start && wxIsspace ( *itr ) );
+
+		size_t endPos = itr - start;
+		if ( endPos > commentEnd.length() )
+		{
+			endPos = itr - text.begin() - commentEnd.length() + 1;
+			ret = text.compare ( endPos, commentEnd.length(), commentEnd );
+
+			// Is commented?
+			if ( ret == 0 )
+			{
+				text.erase ( endPos, commentEnd.length() );
+				text.erase ( startPos, commentStart.length() );
+
+				ReplaceSelection ( text );
+				pos -= commentStart.length();
+				if ( pos >= 0 )
+					SetSelection ( pos, pos );
+				return;
+			}
+		}
+	}
+
+	// Comment selection
+
+	// "--" is not allowed in comments
+	if ( commentStart == _T ( "<!--" ) )
+	{
+		const static wxString doubleHyphen = _T ( "--" );
+		size_t offset = 0;
+		while ( ( offset = text.find ( doubleHyphen, offset ) ) != wxString::npos )
+		{
+			text.replace ( offset, doubleHyphen.length(), _T ( "- -" ) );
+			offset += 2; // WARNING: Not three!
+		}
+	}
+
+	text = commentStart + text + commentEnd;
+
+	ReplaceSelection ( text );
+	if ( pos >= 0 )
+	{
+		pos += commentStart.length();
+		SetSelection ( pos, pos );
+	}
+}
+
+wxString XmlCtrl::getCurrentXPath()
+{
+	wxString xpath;
+	int pos = GetCurrentPos();
+	for ( ; ; )
+	{
+		pos = findPreviousStartTag ( pos, 1, '>' );
+		if ( pos <= 0 )
+			break;
+
+		wxString name = getLastElementName ( pos );
+		size_t pos = name.Index ( ':' );
+		if ( pos != wxString::npos && pos < name.length() )
+		{
+			++pos;
+			name = _T("*[local-name()='")
+					+ name.substr ( pos, name.length() - pos ) + _T("']");
+		}
+		xpath = _T("/") + name + xpath;
+	}
+
+	return xpath;
+}
+
+void XmlCtrl::OnKillFocus ( wxFocusEvent &event )
+{
+	AutoCompCancel();
+	event.Skip();
+}
