@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +30,7 @@
 #include <map>
 
 class ValidationThread;
+class XmlPromptGenerator;
 
 struct XmlCtrlProperties
 {
@@ -109,13 +111,26 @@ class XmlCtrl: public wxStyledTextCtrl
 		    long style = 0 );
 		~XmlCtrl();
 		int getType();
-		int getParentCloseAngleBracket ( int pos, int range = USHRT_MAX * 4 );
+		int getParentCloseAngleBracket ( int pos )
+		{
+			return findPreviousStartTag ( pos, 1, '>' );
+		}
+		int findNextEndTag (
+		    int pos,
+		    unsigned depth = 1,
+		    int character = '>',
+		    int range = USHRT_MAX * 4 );
+		int findPreviousStartTag (
+		    int pos,
+		    unsigned depth = 1,
+		    int character = '<',
+		    int range = USHRT_MAX * 4 );
 		void applyProperties (
 		    const XmlCtrlProperties &propertiesParameter,
 		    bool zoomOnly = false );
 		void applyVisibilityState ( int state = SHOW_TAGS );
 		void updatePromptMaps();
-		void updatePromptMaps ( const char *buffer, size_t bufferLen );
+		void updatePromptMaps ( const char *utf8Buffer, size_t bufferLen );
 		void adjustCursor();
 		void adjustSelection();
 		void foldAll();
@@ -144,8 +159,12 @@ class XmlCtrl: public wxStyledTextCtrl
 		std::string myGetTextRaw(); // alternative to faulty stc implementation
 		bool getValidationRequired();
 		void setValidationRequired ( bool b );
+		bool selectCurrentElement();
+		void toggleComment();
+		wxString getCurrentXPath();
 	private:
 		ValidationThread *validationThread; // used for background validation
+		XmlPromptGenerator *mPromptGeneratorThread;
 
 		int type;
 		bool *protectTags;
@@ -189,6 +208,8 @@ class XmlCtrl: public wxStyledTextCtrl
 		void OnMouseLeftUp ( wxMouseEvent& event );
 		void OnMouseRightUp ( wxMouseEvent& event );
 		void OnMiddleDown ( wxMouseEvent& event );
+		void OnPromptGenerated ( wxNotifyEvent &event );
+		void OnKillFocus ( wxFocusEvent &event );
 		void insertNewLine();
 		void adjustNoColumnWidth();
 		void adjustPosRight();
