@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,13 +19,14 @@
  */
 
 #include <wx/wx.h>
-#include <wx/filename.h>
+#include <wx/tokenzr.h>
 #include <stdexcept>
 #include "xmlpromptgenerator.h"
 #include "xmlencodinghandler.h"
 #include "readfile.h"
 #include "replace.h"
 #include "pathresolver.h"
+#include "threadreaper.h"
 
 #undef XMLCALL
 #include "catalogresolver.h"
@@ -40,14 +42,34 @@
 #include <xercesc/validators/schema/SchemaValidator.hpp>
 #include <xercesc/validators/common/ContentSpecNode.hpp>
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
+#include <xercesc/validators/DTD/DTDGrammar.hpp>
+#include <xercesc/dom/DOMException.hpp>
 
 using namespace xercesc;
 
+extern wxCriticalSection xmlcopyeditorCriticalSection;
+
+DEFINE_EVENT_TYPE ( myEVT_NOTIFY_PROMPT_GENERATED );
+
 XmlPromptGenerator::XmlPromptGenerator (
+	wxEvtHandler *handler,
+	const char *buffer,
+	size_t bufferSize,
     const wxString& basePath,
-    const wxString& auxPath ) : d ( new PromptGeneratorData() )
+    const wxString& auxPath,
+    const char *encoding
+    )
+    : WrapExpat ( encoding )
+    , wxThread ( wxTHREAD_JOINABLE )
+    , d ( new PromptGeneratorData() )
+    , mEventHandler ( handler )
+    , mBuffer ( buffer, bufferSize )
+    , mStopping ( false )
 {
-	XML_SetUserData ( p, d.get() );
+	if ( mBuffer.length() != bufferSize )
+		throw std::bad_alloc();
+
+	XML_SetUserData ( p, this );
 	d->p = p;
 	d->basePath = basePath;
 	d->auxPath = auxPath;
@@ -60,7 +82,7 @@ XmlPromptGenerator::XmlPromptGenerator (
 	XML_SetElementDeclHandler ( p, elementdeclhandler );
 	XML_SetAttlistDeclHandler ( p, attlistdeclhandler );
 	XML_SetEntityDeclHandler ( p, entitydeclhandler );
-	XML_SetExternalEntityRefHandlerArg ( p, d.get() );
+	XML_SetExternalEntityRefHandlerArg ( p, this );
 	XML_SetExternalEntityRefHandler ( p, externalentityrefhandler );
 	XML_SetBase ( p, d->basePath.utf8_str() );
 
@@ -76,18 +98,16 @@ void XMLCALL XmlPromptGenerator::starthandler (
     const XML_Char *el,
     const XML_Char **attr )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data;
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
+	PromptGeneratorData *d = pThis->d.get();
 
 	if (d->isRootElement)
 	{
 		d->rootElement = el;
-		handleSchema ( d, el, attr ); // experimental: schema has been pre-parsed
+		pThis->handleSchema ( d, el, attr ); // experimental: schema has been pre-parsed
 		d->isRootElement = false;
-		if ( ! (d->elementMap.empty() )  )//if ( d->elementMap.size() == 1) // must be 1 for success
+		if ( d->grammarFound || pThis->TestDestroy() )//if ( d->elementMap.size() == 1) // must be 1 for success
 		{
-			d->grammarFound = true;
-			XML_StopParser ( d->p, false );
 			return;
 		}
 	}
@@ -115,45 +135,12 @@ void XMLCALL XmlPromptGenerator::starthandler (
 
 void XMLCALL XmlPromptGenerator::endhandler ( void *data, const XML_Char *el )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data;
-	d->pop();
-}
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
 
-bool XmlPromptGenerator::getGrammarFound()
-{
-	return d->grammarFound;
-}
-
-void XmlPromptGenerator::getAttributeMap (
-    std::map<wxString, std::map<wxString, std::set<wxString> > >
-    &attributeMap )
-{
-	attributeMap = d->attributeMap;
-}
-
-void XmlPromptGenerator::getRequiredAttributeMap (
-    std::map<wxString, std::set<wxString> >& requiredAttributeMap )
-{
-	requiredAttributeMap = d->requiredAttributeMap;
-}
-
-void XmlPromptGenerator::getElementMap (
-    std::map<wxString, std::set<wxString> > &elementMap )
-{
-	elementMap = d->elementMap;
-}
-
-void XmlPromptGenerator::getEntitySet (
-    std::set<wxString> &entitySet )
-{
-	entitySet = d->entitySet;
-}
-
-void XmlPromptGenerator::getElementStructureMap (
-    std::map<wxString, wxString> &elementStructureMap )
-{
-	elementStructureMap = d->elementStructureMap;
+	if ( pThis->TestDestroy() )
+		XML_StopParser ( pThis->d->p, false );
+	else
+		pThis->d->pop();
 }
 
 // handlers for DOCTYPE handling
@@ -171,13 +158,16 @@ void XMLCALL XmlPromptGenerator::doctypedeclstarthandler (
 
 void XMLCALL XmlPromptGenerator::doctypedeclendhandler ( void *data )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data;
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
+	PromptGeneratorData *d = pThis->d.get();
 	if ( !d->elementMap.empty() )
 	{
 		d->grammarFound = true;
 		XML_StopParser ( d->p, false ); // experimental
 	}
+
+	if ( pThis->TestDestroy() )
+		XML_StopParser ( d->p, false );
 }
 
 void XMLCALL XmlPromptGenerator::elementdeclhandler (
@@ -185,14 +175,18 @@ void XMLCALL XmlPromptGenerator::elementdeclhandler (
     const XML_Char *name,
     XML_Content *model )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data;
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
+	PromptGeneratorData *d = pThis->d.get();
 
 	wxString myElement ( name, wxConvUTF8 );
 
-	getContent ( *model, d->elementStructureMap[myElement], d->elementMap[myElement] );
+	pThis->getContent ( *model, d->elementStructureMap[myElement],
+			d->elementMap[myElement] );
 
 	XML_FreeContentModel ( d->p, model );
+
+	if ( pThis->TestDestroy() )
+		XML_StopParser ( d->p, false );
 }
 
 void XmlPromptGenerator::getContent (
@@ -200,6 +194,9 @@ void XmlPromptGenerator::getContent (
     wxString &contentModel,
     std::set<wxString> &list )
 {
+	if ( TestDestroy() )
+		return;
+
 	wxString name;
 	switch ( content.type )
 	{
@@ -256,8 +253,8 @@ void XMLCALL XmlPromptGenerator::attlistdeclhandler (
     const XML_Char *dflt,
     int isrequired )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data;
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
+	PromptGeneratorData *d = pThis->d.get();
 
 	wxString element ( elname, wxConvUTF8 );
 	wxString attribute ( attname, wxConvUTF8 );
@@ -285,6 +282,116 @@ void XMLCALL XmlPromptGenerator::attlistdeclhandler (
 	{
 		d->requiredAttributeMap[element].insert ( attribute );
 	}
+
+	if ( pThis->TestDestroy() )
+		XML_StopParser ( d->p, false );
+}
+
+int XmlPromptGenerator::parseGrammar
+	( PromptGeneratorData *d
+	, const wxString &publicId
+	, const wxString &systemId
+	, const wxString &baseFileName
+	, Grammar::GrammarType type
+	)
+{
+	XercesDOMParser parser;
+	parser.setDoNamespaces ( true );
+	parser.setDoSchema ( true );
+	parser.setValidationSchemaFullChecking ( true );
+
+	MySAX2Handler handler;
+	XercesCatalogResolver catalogResolver;
+	parser.setErrorHandler ( &handler );
+	parser.setEntityResolver ( &catalogResolver );
+
+	Grammar *rootGrammar;
+	try {
+		std::auto_ptr<InputSource> source ( WrapXerces::resolveEntity
+				( publicId , systemId , baseFileName ) );
+		if ( !source.get() )
+		{
+			wxLogError ( _T("Cann't open '%s'"), systemId.c_str() );
+			return XML_STATUS_ERROR;
+		}
+
+		if ( TestDestroy() )
+			return XML_STATUS_ERROR;
+
+		rootGrammar = parser.loadGrammar ( *source, type );
+		if ( !rootGrammar )
+			return XML_STATUS_ERROR;
+	}
+	catch ( SAXParseException &e )
+	{
+		wxLogError ( _T ( "%s" ), handler.getErrors().c_str() );
+		return XML_STATUS_ERROR;
+	}
+	catch ( XMLException &e )
+	{
+		wxString error = WrapXerces::toString ( e.getMessage() );
+		wxLogError ( _T ( "%s" ), error.c_str() );
+		return XML_STATUS_ERROR;
+	}
+	catch ( DOMException &e )
+	{
+		wxString error = WrapXerces::toString ( e.getMessage() );
+		wxLogError ( _T ( "%s" ), error.c_str() );
+		return XML_STATUS_ERROR;
+	}
+	catch (...)
+	{
+		wxLogError ( _T ( "Failed to load: %s %s"), publicId.c_str()
+				, systemId.c_str() );
+		return XML_STATUS_ERROR;
+	}
+
+	wxASSERT ( rootGrammar->getGrammarType() == type );
+
+	if ( type == Grammar::DTDGrammarType )
+	{
+		DTDGrammar* grammar = ( DTDGrammar* ) rootGrammar;
+		NameIdPoolEnumerator<DTDElementDecl> elemEnum
+				= grammar->getElemEnumerator();
+
+		SubstitutionMap substitutions;
+		while ( elemEnum.hasMoreElements() && !TestDestroy() )
+		{
+			const DTDElementDecl& curElem = elemEnum.nextElement();
+			buildElementPrompt ( d, &curElem, substitutions );
+		}
+
+		NameIdPoolEnumerator<DTDEntityDecl>
+				entityEnum = grammar->getEntityEnumerator();
+		while ( entityEnum.hasMoreElements() && !TestDestroy() )
+		{
+			const DTDEntityDecl &entity = entityEnum.nextElement();
+			d->entitySet.insert ( WrapXerces::toString ( entity.getName() ) );
+		}
+	}
+	else
+	{
+		wxASSERT ( type == Grammar::SchemaGrammarType );
+
+		SchemaGrammar* grammar = ( SchemaGrammar* ) rootGrammar;
+		RefHash3KeysIdPoolEnumerator<SchemaElementDecl> elemEnum = grammar->getElemEnumerator();
+
+		if ( !elemEnum.hasMoreElements() )
+		{
+			return XML_STATUS_ERROR;
+		}
+
+		SubstitutionMap substitutions;
+		buildSubstitutionMap ( substitutions, *grammar );
+
+		while ( elemEnum.hasMoreElements() && !TestDestroy() )
+		{
+			const SchemaElementDecl& curElem = elemEnum.nextElement();
+			buildElementPrompt ( d, &curElem, substitutions );
+		}
+	}
+
+	return TestDestroy() ? XML_STATUS_ERROR : XML_STATUS_OK;
 }
 
 int XMLCALL XmlPromptGenerator::externalentityrefhandler (
@@ -294,8 +401,9 @@ int XMLCALL XmlPromptGenerator::externalentityrefhandler (
     const XML_Char *systemId,
     const XML_Char *publicId )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) p; // arg is set to user data in c'tor
+	// arg is set to user data in c'tor
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) p;
+	PromptGeneratorData *d = pThis->d.get();
 
 	int ret;
 	std::string buffer;
@@ -311,7 +419,7 @@ int XMLCALL XmlPromptGenerator::externalentityrefhandler (
 
 		d->encoding = XmlEncodingHandler::get ( buffer );
 		XML_Parser dtdParser = XML_ExternalEntityParserCreate ( d->p, context, d->encoding.c_str() );
-		if ( !dtdParser )
+		if ( pThis->TestDestroy() || !dtdParser )
 			return XML_STATUS_ERROR;
 		XML_SetBase ( dtdParser, d->auxPath.utf8_str() );
 		ret = XML_Parse ( dtdParser, buffer.c_str(), buffer.size(), true );
@@ -319,6 +427,20 @@ int XMLCALL XmlPromptGenerator::externalentityrefhandler (
 		return ret;
 	}
 
+	// Either EXPAT or Xerces-C++ is capable of parsing DTDs. The advantage
+	// of Xerces-C++ is that it can access DTDs that are on the internet.
+#if !PREFER_EXPAT_TO_XERCES_C
+
+	return pThis->parseGrammar
+			( d
+			, wxString::FromUTF8 ( publicId )
+			, wxString::FromUTF8 ( systemId )
+			, d->basePath
+			, Grammar::DTDGrammarType
+			);
+
+#else // !PREFER_EXPAT_TO_XERCES_C
+
 	wxString widePublicId ( publicId, wxConvUTF8 );
 	wxString wideSystemId ( systemId, wxConvUTF8 );
 	CatalogResolver cr;
@@ -348,14 +470,17 @@ int XMLCALL XmlPromptGenerator::externalentityrefhandler (
 
 	std::string encoding = XmlEncodingHandler::get ( buffer );
 	XML_Parser dtdParser = XML_ExternalEntityParserCreate ( d->p, context, encoding.c_str() );
-	if ( !dtdParser )
+	if ( pThis->TestDestroy() || !dtdParser )
 		return XML_STATUS_ERROR;
 
 	XML_SetBase ( dtdParser, wideSystemId.utf8_str() );
 
 	ret = XML_Parse ( dtdParser, buffer.c_str(), buffer.size(), true );
 	XML_ParserFree ( dtdParser );
-	return ret;
+
+	return pThis->TestDestroy() ? XML_STATUS_ERROR : ret;
+
+#endif // PREFER_EXPAT_TO_XERCES_C
 }
 
 void XMLCALL XmlPromptGenerator::entitydeclhandler (
@@ -369,8 +494,9 @@ void XMLCALL XmlPromptGenerator::entitydeclhandler (
     const XML_Char *publicId,
     const XML_Char *notationName )
 {
-	PromptGeneratorData *d;
-	d = ( PromptGeneratorData * ) data; // arg is set to user data in c'tor
+	// arg is set to user data in c'tor
+	XmlPromptGenerator *pThis = ( XmlPromptGenerator * ) data;
+	PromptGeneratorData *d = pThis->d.get();
 
 	if (
 	    entityName &&
@@ -381,6 +507,9 @@ void XMLCALL XmlPromptGenerator::entitydeclhandler (
 	{
 		d->entitySet.insert ( wxString ( entityName, wxConvUTF8 ) );
 	}
+
+	if ( pThis->TestDestroy() )
+		XML_StopParser ( d->p, false );
 }
 
 void XmlPromptGenerator::handleSchema (
@@ -390,118 +519,92 @@ void XmlPromptGenerator::handleSchema (
 {
 	if ( !d->isRootElement )
 		return;
+
+	wxString publicId, systemId;
+
 	// first check for XML Schema association
 	const char **schemaAttr = ( const char ** ) attr; // now redundant; could use attr
-	wxString path;
 	for ( ; *schemaAttr; schemaAttr += 2 )
 	{
 		// no namespace
 		if ( !strcmp ( *schemaAttr, "xsi:noNamespaceSchemaLocation" ) )
 		{
-			path = wxString ( schemaAttr[1], wxConvUTF8 );
+			systemId = wxString ( schemaAttr[1], wxConvUTF8 );
 			break;
 		}
 		// with namespace -- check if this works
 		else if ( !strcmp ( *schemaAttr, "xsi:schemaLocation" ) )
 		{
-			const char *searchIterator = * ( schemaAttr + 1 );
-			while ( *searchIterator && *searchIterator != ' ' && *searchIterator != '\t' && *searchIterator != '\n' )
-				searchIterator++;
-			if ( *searchIterator )
-			{
-				path = wxString ( searchIterator + 1, wxConvUTF8 );
-				break;
-			}
+			wxStringTokenizer tokens ( wxString ( schemaAttr[1], wxConvUTF8 ) );
+			size_t count = tokens.CountTokens();
+			if ( count < 2 )
+				return;
+			// TODO: Support multiple schemas
+			publicId = tokens.GetNextToken();
+			systemId = tokens.GetNextToken();
 		}
 	}
 
-	if ( path.empty() )
-	{
+	int ret = parseGrammar ( d, publicId, systemId, d->basePath
+			, Grammar::SchemaGrammarType );
+	if ( ret != XML_STATUS_OK )
 		return;
+
+	d->grammarFound = true;
+	XML_StopParser ( d->p, false );
+}
+
+void XmlPromptGenerator::buildElementPrompt (
+    PromptGeneratorData *d,
+    const XMLElementDecl *element,
+    SubstitutionMap &substitutions )
+{
+
+	wxString name;
+
+	const QName *qnm = element->getElementName();
+	if ( qnm == NULL )
+		return;
+	name = WrapXerces::toString ( qnm->getRawName() ); // this includes any prefix:localname combinations
+	if ( name.empty() )
+		return;
+
+	const XMLCh* fmtCntModel = element->getFormattedContentModel();
+	if ( fmtCntModel != NULL ) // tbd: this does not yet pick up prefix:localname combinations
+	{
+		wxString structure = WrapXerces::toString ( fmtCntModel );
+		d->elementStructureMap[name] = structure;
+	}
+	const ContentSpecNode *spec = element->getContentSpec();
+	if ( spec != NULL )
+	{
+		getContent ( d->elementMap[name], spec, substitutions );
 	}
 
-
-	wxString schemaPath = PathResolver::run ( path, ( d->auxPath.empty() ) ? d->basePath : d->auxPath);
-
-	std::auto_ptr<XercesDOMParser> parser ( new XercesDOMParser() );
-	parser->setDoNamespaces ( true );
-	parser->setDoSchema ( true );
-	parser->setValidationSchemaFullChecking ( true );
-
-	Grammar *rootGrammar = parser->loadGrammar
-			( ( const XMLCh * ) WrapXerces::toString ( schemaPath ).GetData()
-			, Grammar::SchemaGrammarType
-			);
-	if ( !rootGrammar )
-	{
+	// fetch attributes
+	if ( !element->hasAttDefs() )
 		return;
-	}
 
-	SchemaGrammar* grammar = ( SchemaGrammar* ) rootGrammar;
-	RefHash3KeysIdPoolEnumerator<SchemaElementDecl> elemEnum = grammar->getElemEnumerator();
-
-	if ( !elemEnum.hasMoreElements() )
+	XMLAttDefList& attIter = element->getAttDefList();
+	unsigned int i = 0;
+	for ( ; i < attIter.getAttDefCount() && !TestDestroy(); i++ )
 	{
-		return;
-	}
+		wxString attribute, attributeValue;
 
-	SubstitutionMap substitutions;
-	buildSubstitutionMap ( substitutions, *grammar );
-
-	while ( elemEnum.hasMoreElements() )
-	{
-		const SchemaElementDecl& curElem = elemEnum.nextElement();
-
-		wxString element;
-
-		const QName *qnm = curElem.getElementName();
-		if ( qnm == NULL )
+		XMLAttDef& attr = attIter.getAttDef ( i );
+		XMLAttDef::DefAttTypes ty = attr.getDefaultType();
+		if ( ty == XMLAttDef::Prohibited )
 			continue;
-		element = WrapXerces::toString ( qnm->getRawName() ); // this includes any prefix:localname combinations
-		if ( element.empty() )
+		attribute = WrapXerces::toString ( attr.getFullName() );
+		if ( attribute.empty() )
 			continue;
 
-		const XMLCh* fmtCntModel = curElem.getFormattedContentModel();
-		if ( fmtCntModel != NULL ) // tbd: this does not yet pick up prefix:localname combinations
-		{
-			wxString structure = WrapXerces::toString ( fmtCntModel );
-			d->elementStructureMap[element] = structure;
-		}
-		const ContentSpecNode *spec = curElem.getContentSpec();
-		if ( spec != NULL )
-		{
-			getContent ( d->elementMap[element], spec, substitutions );
-		}
+		// Value
+		attributeValue = WrapXerces::toString ( attr.getValue() );
+		d->attributeMap[name][attribute].insert( attributeValue );
 
-		// fetch attributes
-		if ( !curElem.hasAttDefs() )
-			continue;
-
-		XMLAttDefList& attIter = curElem.getAttDefList();
-		for ( unsigned int i = 0; i < attIter.getAttDefCount(); i++ )
-		{
-			wxString attribute, attributeValue;
-
-			XMLAttDef& attr = attIter.getAttDef ( i );
-			XMLAttDef::DefAttTypes ty = attr.getDefaultType();
-			if ( ty == XMLAttDef::Prohibited )
-				continue;
-			SchemaAttDef *pAttr = ( SchemaAttDef * ) &attr;
-
-			const QName *qnm = pAttr->getAttName();
-			if ( qnm == NULL )
-				continue;
-			attribute = WrapXerces::toString ( qnm->getRawName() );
-			if ( attribute.empty() )
-				continue;
-
-			// Value
-			attributeValue = WrapXerces::toString ( pAttr->getValue() );
-			d->attributeMap[element][attribute].insert( attributeValue );
-
-			if ( ty == XMLAttDef::Required || ty == XMLAttDef::Required_And_Fixed)
-				d->requiredAttributeMap[element].insert ( attribute );
-		}
+		if ( ty == XMLAttDef::Required || ty == XMLAttDef::Required_And_Fixed)
+			d->requiredAttributeMap[name].insert ( attribute );
 	}
 }
 
@@ -515,7 +618,7 @@ void XmlPromptGenerator::buildSubstitutionMap (
 	if ( !list.hasMoreElements() )
 		return;
 
-	while ( list.hasMoreElements() )
+	while ( list.hasMoreElements() && !TestDestroy() )
 	{
 		const ElemVector &elmts = list.nextElement();
 
@@ -540,6 +643,8 @@ void XmlPromptGenerator::getContent (
     const ContentSpecNode *spec,
     SubstitutionMap &substitutions )
 {
+	if ( TestDestroy() )
+		return;
 	//if ( spec == NULL) return;
 
 	const QName *qnm = spec->getElement();
@@ -566,3 +671,28 @@ void XmlPromptGenerator::getContent (
 	if ( spec->getSecond() != NULL)
 		getContent( list, spec->getSecond(), substitutions );
 }
+
+void *XmlPromptGenerator::Entry()
+{
+	if ( TestDestroy() )
+		return NULL;
+
+	parse ( mBuffer );
+
+	wxCriticalSectionLocker locker ( xmlcopyeditorCriticalSection );
+
+	if ( !TestDestroy() )
+	{
+		wxNotifyEvent event ( myEVT_NOTIFY_PROMPT_GENERATED );
+		wxPostEvent ( mEventHandler, event );
+	}
+
+	return NULL;
+}
+
+void XmlPromptGenerator::PendingDelete ()
+{
+	Cancel();
+
+	ThreadReaper::get().add ( this );
+}
