@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,7 +35,7 @@ struct SchemaLocatorData
 class XmlSchemaLocator : public WrapExpat
 {
 	public:
-		XmlSchemaLocator();
+		XmlSchemaLocator ( const char *encoding );
 		virtual ~XmlSchemaLocator();
 		std::string getSchemaLocation();
 	private:
