@@ -3,9 +3,10 @@
  *
  * This file is part of Xml Copy Editor.
  *
- * Xml Copy Editor is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * Xml Copy Editor is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * Xml Copy Editor is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +23,9 @@
 #include "xmlwordcount.h"
 #include "getword.h"
 
-XmlWordCount::XmlWordCount() : wcd ( new WordCountData() )
+XmlWordCount::XmlWordCount ( const char *encoding )
+	: WrapExpat ( encoding, false )
+	, wcd ( new WordCountData() )
 {
 	wcd->wordCount = 0;
 	XML_SetUserData ( p, wcd.get() );
